package com.isu.gaswellwatch.vo.command.scss;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.math.BigDecimal;
import java.util.Collection;
import java.util.List;

/**
 * 采样间隔
 *
 * @author <a href="mailto:shilong.wang@alpha-ess.com">王仕龙</a>
 * 2024/12/10 17:24
 */
@Getter
@Setter
@SuperBuilder
public class SamplingInterval extends Command {

    @Serial
    private static final long serialVersionUID = -649748614798682095L;

    /**
     * 连续采样间隔
     */
    @NotNull(message = "连续采样间隔不能为空且需大于0")
    private int continuousSamplingIntervalDuration;
    /**
     * 到达传感器有效电平
     */
    @NotNull(message = "到达传感器有效电平不能为空且需大于0")
    private BigDecimal sensorSignalEffectiveLevel;

    public SamplingInterval() {
        this.setCode("SCSS.SAMPLING_INTERVAL");
    }

    @Override
    public boolean validate() {
        return super.validate()
                && this.continuousSamplingIntervalDuration > 0
                && this.sensorSignalEffectiveLevel.compareTo(BigDecimal.ZERO) > 0;
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(250);
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0054    0004     08
        command.append("01100054000408");
        command.append(StringUtils.leftPad(Integer.toHexString(this.continuousSamplingIntervalDuration), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.sensorSignalEffectiveLevel.multiply(ONE_HUNDRED).intValue()), 8, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }
}
